#!/usr/bin/perl -w

# LSB Test Execution Framework
# Configuration Save Module (tests_conf_save.pl)
#
# Copyright (C) 2007 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

use Templates;
use Fcntl ':flock';

sub is_process_finished($) {
	my $PID = $_[0];
	my $test_line = `ps -p $PID -o s --no-heading`;
	$test_line =~ s/[\r\n]//g;
	return ($test_line !~ m/^[^Zz]/);
}

my $conf_page = ($_POST{"certify"} ? "tests_cert.pl" : "tests_conf.pl");
if ($_GET{"goto"}) {
	$conf_page = $_GET{"goto"};
	$conf_page =~ s/%23/#/;
	$_POST{"action"} = "";
}
my $session_profile = ($_COOKIE{"session_id"} ? "~session.$PeerIP.".$_COOKIE{"session_id"} : "");

my $error;
my $new_url = "";
my $save_profile_name = "";

if (!-d "$APP_DATA/profiles") {
	if (!mkdir("$APP_DATA/profiles")) {
		$error = "";
	}
}

if (!defined($error)) {
	if ((!$_POST{"action"} or ($_POST{"action"} eq "")) and (!$_GET{"goto"})) {
		$error = "Script action not specified!";
	}
	elsif ($_POST{"action"} eq "load_profile") {
		if (!$_POST{"edit_profile_name"}) {
			$error = "No profile name specified!";
		}
		elsif ($_POST{"edit_profile_name"} =~ m/[\\\/]/) {
			$error = "Invalid profile name!";
		}
		elsif (!-f "$APP_DATA/profiles/".$_POST{"edit_profile_name"}) {
			$error = "File not found:<br />$APP_DATA/profiles/".$_POST{"edit_profile_name"};
		}
		else {
			system("cp \"$APP_DATA/profiles/$_POST{edit_profile_name}\" \"$APP_DATA/profiles/$session_profile\" > /dev/null");
			$new_url = $conf_page;
		}
	}
	elsif ($_POST{"action"} eq "refresh_list") {
		$new_url = "tests_conf_refresh.pl";
		if ($_POST{"certify"}) {
			$new_url .= "?cert=".$_POST{"certify"};
		}
		$save_profile_name = $session_profile;
	}
	elsif ($_POST{"action"} eq "run_tests") {
		my $tests_exec = 0;
		if (-f "/tmp/tests_output.8888") {
			if (open(FILE, "< /tmp/tests_output.8888")) {
				my $PID = <FILE>;
				$PID =~ m/PID: (\d+),.*/;
				$PID = $1;
				if (is_process_finished($PID) and flock(FILE, LOCK_EX | LOCK_NB)) {
#					unlink("/tmp/tests_output.8888");
					rename("/tmp/tests_output.8888", "/tmp/tests_output.8888.old");
					flock(FILE, LOCK_UN);
					$tests_exec = 1;
				}
				close(FILE);
			}
		}
		else {
			$tests_exec = 1;
		}

		if (!$tests_exec) {
			$save_profile_name = $session_profile;
			$error = "Tests are already running!<br />You cannot run another instance before they are finished.<br />You can watch or stop the running tests on the <a href=\"tests_exec.pl\"><u>Execution</u></a> page.";
		}
		else {
			if ($session_profile eq "") {
				$error = "Cannot retrieve session ID! Please, allow cookies in your browser.";
			}
			else {
				$new_url = "tests_exec.pl?profile=$session_profile";
				if ($_POST{"certify"}) {
					$new_url .= "&cert=".$_POST{"certify"};
				}
				$save_profile_name = $session_profile;
			}
		}
	}
	elsif ($_POST{"action"} eq "save_profile") {
		if (!$_POST{"edit_profile_name"}) {
			$error = "No profile name specified!";
		}
		elsif ($_POST{"edit_profile_name"} =~ m/[\\\/]/) {
			$error = "Invalid profile name!";
		}
		else {
			$save_profile_name = $_POST{"edit_profile_name"};
			$new_url = $conf_page;
		}
	}
	elsif ($_POST{"action"} eq "delete_profile") {
		if (!$_POST{"edit_profile_name"}) {
			$error = "No profile name specified!";
		}
		elsif ($_POST{"edit_profile_name"} =~ m/[\\\/]/) {
			$error = "Invalid profile name!";
		}
		elsif (!-f "$APP_DATA/profiles/".$_POST{"edit_profile_name"}) {
			$error = "File not found:<br />$APP_DATA/profiles/".$_POST{"edit_profile_name"};
		}
		else {
			unlink("$APP_DATA/profiles/".$_POST{"edit_profile_name"});
			$save_profile_name = $session_profile;
			$new_url = $conf_page;
		}
	}
	elsif ($_GET{"goto"}) {
		$_POST{"action"} = "";
		$save_profile_name = $session_profile;
		$new_url = $conf_page;
	}
}

if (!defined($error)) {
	if ($save_profile_name ne "") {
		my $full_save_name = "$APP_DATA/profiles/$save_profile_name";
		if (open(FILE, "> $full_save_name")) {
			my $architecture = ($_POST{"architecture"} or "ia32");
			my $lsb_version = ($_POST{"lsb_version"} or "3.1");
			my $lsb_profile = ($_POST{"lsb_profile"} or "desktop");
			print FILE "CERTIFY:         ".$_POST{"certify"}."\n"      if ($_POST{"certify"});
			print FILE "NAME:            ".$_POST{"tester_name"}."\n"  if ($_POST{"tester_name"});
			print FILE "ORGANIZATION:    ".$_POST{"organization"}."\n" if ($_POST{"organization"});
			if ($_POST{"tester_email"}) {
				print FILE "EMAIL:           ".$_POST{"tester_email"}."\n";
				print FILE "SEND_EMAIL:      ".($_POST{"send_email_report"} ? "1" : "0")."\n";
			}

			print FILE "VERBOSE_LEVEL:   ".($_POST{"verbose_level"} ? $_POST{"verbose_level"} : "1")."\n";
			print FILE "KEEP_PACKAGES:   ".($_POST{"keep_packages"} ? "1" : "0")."\n";
			print FILE "SHORT_SCENARIOS: ".($_POST{"short_scen"} ? "1" : "0")."\n";
			print FILE "ARCHITECTURE:    $architecture\n";
			print FILE "USE_INTERNET:    ".($_POST{"use_internet"} ? "1" : "0")."\n";
			print FILE "LSB_VERSION:     $lsb_version\n";
			print FILE "LSB_PROFILE:     $lsb_profile\n";

			if ($_POST{"comments"}) {
				$_POST{"comments"} =~ s/\r\n/\n/g;
				$_POST{"comments"} =~ s/&/&amp;/g;
				$_POST{"comments"} =~ s/</&lt;/g;
				$_POST{"comments"} =~ s/>/&gt;/g;
				my @comments_lines = split(/\n/, $_POST{"comments"});
				foreach (@comments_lines) {
					print FILE "COMMENTS: $_\n";
				}
			}

			foreach (sort grep /mod-$architecture-$lsb_version-/, keys %_POST) {
				my $tmp = $_;
				$tmp =~ s/^mod-/modstage-/;
				s/^mod-$architecture-$lsb_version-//;
				print FILE "$_ ".$_POST{$tmp}."\n";
				$tmp =~ s/^modstage-/modopt-/;
				if ($_POST{$tmp}) {
					print FILE "$_-opt ".$_POST{$tmp}."\n";
				}
			}

			close(FILE);

			if ($_POST{"action"} eq "save_profile") {
				system("cp \"$full_save_name\" \"$APP_DATA/profiles/$session_profile\" > /dev/null");
			}
			elsif ($_POST{"action"} eq "run_tests") {
				if (open(CHK_FILE, $CONFIG{"TESTS_DIR"}."/lsb-tef.pl --check-only -f \"$full_save_name\" |")) {
					$error = "";
					while(<CHK_FILE>) {
						s/[\r\n]//g;
						if (m/^ERROR: (.*)/i) {
							$error .= ($error ? "<br />" : "").$1;
						}
					}
					close(CHK_FILE);
					undef $error if ($error eq "");
					if (defined($error)) {
						$error .= "<br />Please, see the <a href=\"tests_help.pl#troubleshooting\"><u>Troubleshooting</u></a> Help section to find the solution.";
						$error .= "<br />If you could not find any, please, write at <a href=\"mailto:fsgteam\@ispras.ru\"><u>fsgteam\@ispras.ru</u></a>.";
					}
				}
				else {
					$error = "Cannot run lsb-tef.pl to check the requirements!<br />$!";
				}
			}
		}
		else {
			$error = "Cannot open file $save_profile_name for writing!<br />$!";
		}
	}
}

if (defined($error)) {
	escape($error);
	print "HTTP/1.0 302 Moved\n";
	print "Set-Cookie: last_error_text=$error; path=/;\n";
	print "Location: $conf_page\n\n";
}
else {
	print "HTTP/1.0 302 Moved\n";
	print "Location: $new_url\n\n";
}
